package com.chelvc.cloud.vehicle.server.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.chelvc.cloud.uc.api.dto.UserDTO;
import com.chelvc.cloud.uc.api.service.UserService;
import com.chelvc.cloud.vehicle.api.dto.RankAwardRecordDTO;
import com.chelvc.cloud.vehicle.api.param.HandRankAwardParam;
import com.chelvc.cloud.vehicle.api.param.QueryRankAwardRecordParam;
import com.chelvc.cloud.vehicle.api.service.RankAwardRecordService;
import com.chelvc.cloud.vehicle.server.dao.RankAwardRecordMapper;
import com.chelvc.cloud.vehicle.server.entity.RankAwardConfig;
import com.chelvc.cloud.vehicle.server.entity.RankAwardRecord;
import com.chelvc.cloud.vehicle.server.service.RankAwardConfigService;
import com.chelvc.framework.base.exception.ResourceUnavailableException;
import com.chelvc.framework.common.model.Pagination;
import lombok.RequiredArgsConstructor;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.dubbo.config.annotation.DubboReference;
import org.apache.dubbo.config.annotation.DubboService;
import org.springframework.beans.factory.annotation.Autowired;

import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@RequiredArgsConstructor(onConstructor = @__(@Autowired))
@DubboService(interfaceClass = com.chelvc.cloud.vehicle.api.service.RankAwardRecordService.class)
public class RankAwardRecordServiceImpl extends ServiceImpl<RankAwardRecordMapper, RankAwardRecord> implements RankAwardRecordService {

    @DubboReference
    UserService userService;
    private final RankAwardConfigService rankAwardConfigService;

    @Override
    public Pagination<RankAwardRecordDTO> queryPageList(QueryRankAwardRecordParam param, Long pageNum, Long pageSize) {
        Page<RankAwardRecordDTO> page = new Page<>(pageNum, pageSize);
        QueryWrapper<RankAwardConfig> wrapper = Wrappers.query();
        wrapper.eq(param.getType() != null, "r.type", param.getType());
        wrapper.eq(param.getPlatformType() != null, "r.platform_type", param.getPlatformType());
        wrapper.eq(param.getPeriod() != null, "r.period", param.getPeriod());
        wrapper.orderByDesc("r.create_time");
        IPage<RankAwardRecordDTO> result = baseMapper.queryPageList(page, wrapper);
        List<RankAwardRecordDTO> records = result.getRecords();
        if(!CollectionUtils.isEmpty(records) && param.getPlatformType() != null && param.getPlatformType() !=2){
            List<Long> userIds = records.stream().filter(e -> e.getPlatformType() != 2).map(RankAwardRecordDTO::getUserId).collect(Collectors.toList());
            if(!CollectionUtils.isEmpty(userIds)){
                Map<Long, UserDTO> userMap = userService.queryUserByIds(userIds);
                if(userMap != null){
                    for(RankAwardRecordDTO dto : records){
                        Integer platformType = dto.getPlatformType();
                        if(platformType != 2){
                            UserDTO userDTO = userMap.get(dto.getUserId());
                            dto.setName(userDTO == null ? "" : userDTO.getNickname());
                        }
                    }
                }
            }
        }
        return Pagination.<RankAwardRecordDTO>builder().total(result.getTotal()).pages(result.getPages())
                .records(result.getRecords()).build();
    }

    @Override
    public void handOutPrize(HandRankAwardParam param) {
        Long awardId = param.getAwardId();
        RankAwardConfig config = rankAwardConfigService.queryById(awardId);
        if(config == null){
            throw new ResourceUnavailableException("参数错误");
        }
        Integer period = param.getPeriod();
        Long userId = param.getUserId();
        LambdaQueryWrapper<RankAwardRecord> wrapper = Wrappers.lambdaQuery();
        wrapper.eq(RankAwardRecord::getUserId, userId);
        wrapper.eq(RankAwardRecord::getType, config.getType());
        wrapper.eq(RankAwardRecord::getPlatformType, config.getPlatformType());
        wrapper.eq(RankAwardRecord::getPeriod, period);
        List<RankAwardRecord> rankAwardRecords = baseMapper.selectList(wrapper);
        if(!CollectionUtils.isEmpty(rankAwardRecords)){
            throw new ResourceUnavailableException("本次奖励已发放");
        }
        //TODO  发放奖励

        RankAwardRecord record = new RankAwardRecord();
        record.setUserId(userId);
        //TODO  比例转成具体金额
        record.setBonus(new BigDecimal(config.getRatio()));
        record.setRegionLevel(config.getRegionLevel());
        record.setPeriod(period);
        record.setType(config.getType());
        record.setPlatformType(config.getPlatformType());
        record.setAwardTime(LocalDateTime.now());
        record.setCreateTime(LocalDateTime.now());
        baseMapper.insert(record);
    }

    @Override
    public void awardRank() {

    }
}
